%%
%%  wpc_stereoscopic.erl --
%%
%%     Stereoscopic Geometry Windows Plug-in
%%  Copyright (c) 2013
%%  Original source patch for wings.erl, wings_camera.erl,
%%   wings_view.erl, wings_wm.erl by Damian Frey on 13 Apr 2010.
%%  Converted to plug-in by Wings3D User oort (Bernard Oortman)
%%
%%     $Id$
%%

-module(wpc_stereoscopic).

%% wings.erl stuff added by OORT start
-define(NEED_OPENGL, 1).
-define(NEED_ESDL, 1).
%% wings.erl stuff added by OORT end

-include("wings.hrl").

-export([init/0,menu/2,command/2]).
%% WINGS_WM.erl Original Patch Section 1 Start
%% stereo stuff
-export([has_stereo_link/0,has_stereo_link/1,get_stereo_link/0,get_stereo_link/1,stereo_link/2]).
%% WINGS_WM.erl Original Patch Section 1 end

-import(math, [cos/1,sin/1,pi/0]).
%% Add Imports by OORT start
-import(wings, [new_viewer/6]).
%-import(wings_wm, [toplevel/0,toplevel/6]).
%% Add Imports by OORT end

%% wings_wm.erl stuff added by OORT start
-record(win,
	{z,					    %Z order.
	 x,y,					%Position.
	 w,h,					%Size.
	 name,					%Name of window.
	 stk,					%Event handler stack.
	 props,					%Window properties.
	 links=[],			    %Windows linked to this one.
	 rollup=false			%Rollup window into Title Bar {controller,Name}
	}).
%% wings_wm.erl stuff added by OORT end


%% Define default values Start, added by OORT
-define(DEF_CROSS_EYED, true).
%-define(DEF_PAN_X, 0.07).
%% Define default values End, added by OORT


%% Value ranges Start, added by OORT
%range(T) -> {range,range_1(T)}.
%range_1(pan_x)	-> {-0.14,0.14}.
%% Value ranges End, added by OORT



%% Remember Prefs Start, added by OORT
init() ->
    wings_pref:set_default(cross_eyed, ?DEF_CROSS_EYED),    
%%    wpa:pref_set_default(?MODULE, pan_x, ?DEF_PAN_X),

%% Add WPC_NUMERIC_CAMERA.erl stuff by OORT Start (Adds info to preferences.txt may not be needed)    
%    wings_pref:set_default(cam_pos_specify_dist,false),
%    wings_pref:set_default(cam_pos_preview,true),
%% Add WPC_NUMERIC_CAMERA.erl stuff by OORT End.    
    true.
%% Remember Prefs End, added by OORT


%% Insert Menu items start, added by OORT
menu({window}, Menu) ->
    Menu ++ [separator|stereo_menu()];
menu(_, Menu) -> Menu.

stereo_menu() ->
    [
%% WINGS.erl Original Patch Section 3 Start, listed for reference.
%%+ {"New Geometry Window stereo",geom_viewer_stereo, "Open a new stereo Geometry window"},
%% WINGS.erl Original Patch Section 3 End, listed for reference.    
   {?__(1,"Stereoscopic Windows"),geom_viewer_stereo,[option]}].
%% Insert Menu items end, added by OORT
 


%% WINGS.erl Original Patch Section 2 start, listed for reference.
%%+command_1({window,geom_viewer_stereo}, St) ->
%%+	new_viewer_stereo(St),
%%+	keep;
%% WINGS.erl Original Patch Section 2 end, listed for reference.

%% WINGS.erl Modified Patch Section 2 Start
command({window,{geom_viewer_stereo, Arg}},St) -> new_viewer_stereo(Arg,St),
	keep;
%% WINGS.erl Modified Patch Section 2 End	


command(_, _) -> next.


%% WINGS.erl Original Patch Section 1 Start, listed for Reference
%%+new_viewer_stereo(St) ->
%%+    {Pos,{W,H}} = wings_wm:win_rect(desktop),
%%+    Size = {W div 2-40,H div 2-40},
%%+    N = free_viewer_num(2),
%%+    Active = wings_wm:this(),
%%+    Props = wings_wm:get_props(Active),
%%+    ToolbarHidden = wings_wm:is_hidden({toolbar,Active}),
%%+    Name = {geom,N},
%%+    new_viewer(Name, Pos, Size, Props, ToolbarHidden, St),
%%+	% second window
%%+	N2 = free_viewer_num(2),
%%+	Name2 = {geom,N2},
%%+	{X2,Y2} = Pos,
%%+	new_viewer(Name2, {X2+element(1,Size),Y2}, Size, Props, ToolbarHidden, St),
%%+	% now link
%%+	wings_wm:stereo_link(Name,Name2).
%% WINGS.erl Original Patch Section 1 End, listed for Reference


%% Start stereoscopic_dialog modified to add use of defined default values, added by OORT
stereoscopic_dialog() ->
    Cross_Eyed = get_pref(cross_eyed, ?DEF_CROSS_EYED),
    [{vradio, [{?__(1,"On"), true},
	       {?__(2,"Off"), false}],
	      Cross_Eyed,
	      [{key,cross_eyed}, {title, ?__(3,"Cross Eyed Viewing")}]}
    ].
%% End stereoscopic_dialog modified to add use of defined default values, added by OORT


%% New code to add stereoscopic option dialog start
new_viewer_stereo(Arg,_St) when is_atom(Arg) ->
    wpa:dialog(Arg, ?__(1,"Stereoscopic Options"), stereoscopic_dialog(),
	fun(Res) -> {window,{geom_viewer_stereo,Res}} end);	
%% New code to add stereoscopic option dialog end


%% WINGS.erl Modified Patch Section 1 Start (Code Creates Two Geometry Windows)
%% This gets exported first.
new_viewer_stereo(Arg,St) ->
{Pos,{W,H}} = wings_wm:win_rect(desktop),
    Size = {W div 2-40,H div 2-40},
    N = free_viewer_num(2),
    Active = wings_wm:this(),
    Props = get_props(Active),
    ToolbarHidden = wings_wm:is_hidden({toolbar,Active}),
    Name = {geom,N},
    new_viewer(Name, Pos, Size, Props, ToolbarHidden, St),
	% second window
	N2 = free_viewer_num(2),
	Name2 = {geom,N2},
	{X2,Y2} = Pos,
	new_viewer(Name2, {X2+element(1,Size),Y2}, Size, Props, ToolbarHidden, St),
	% now link
%% WINGS.erl Modified Patch Section 1 End (Code Creates Two Geometry Windows)

%% Check if Cross-eyed start
    Cross_Eyed = proplists:get_value(cross_eyed, Arg, ?DEF_CROSS_EYED),
	case Cross_Eyed of
		true ->
	stereo_link(Name2,Name),
			io:format("Cross-eyed Mode ON\n");	
		false ->
	stereo_link(Name,Name2),
			io:format("Cross-eyed Mode OFF\n")	
	end.
%% Check if Cross-eyed end

%% wings.erl missing stuff added start, by oort (code needed for new_viewer_stereo)
free_viewer_num(N) ->
    case wings_wm:is_window({geom,N}) of
	false -> N;
	true -> free_viewer_num(N+1)
    end.
%% wings.erl missing stuff added end, by oort (code needed for new_viewer_stereo)



%% Add more WINGS_WM.erl stuff added start for New_Viewer_Stereo Code
get_props(Win) ->
    #win{props=Props} = get_window_data(Win),
    gb_trees:to_list(Props).
%% Add more WINGS_WM.erl stuff added end for New_Viewer_Stereo Code


%% Add more WINGS_WM.erl stuff added start for Stereo_link Code
get_prop(Win, Name) ->
    #win{props=Props} = get_window_data(Win),
    gb_trees:get(Name, Props).
%% Add more WINGS_WM.erl stuff added end for Stereo_link Code

%% Add more WINGS_WM.erl stuff added start for Stereo_Link Code
lookup_prop(Win, Name) ->
    #win{props=Props} = get_window_data(Win),
    gb_trees:lookup(Name, Props).
%% Add more WINGS_WM.erl stuff added end for Stereo_Link Code


%% WINGS.erl New_Viewer stuff start, listed for reference

%new_viewer(Name, {X,Y}, Size, Props, ToolbarHidden, St) ->
%    Op = main_loop_noredraw(St),
%    Title = geom_title(Name),
%    wings_wm:toplevel(Name, Title, {X,Y,highest}, Size,
%		      [resizable,closable,{anchor,nw},
%		       {toolbar,fun(A, B, C) ->
%					wings_toolbar:create(A, B, C)
%				end},
%		       menubar,
%		       {properties,Props}],
%		      Op),
%    wings_wm:menubar(Name, get(wings_menu_template)),
%    wings_wm:send({menubar,Name}, {current_state,St}),
%    wings_wm:send({toolbar,Name}, {current_state,St}),
%    set_drag_filter(Name),
%    if
%	ToolbarHidden -> wings_wm:hide({toolbar,Name});
%	true -> ok
%    end,
%    Name.

%% WINGS.erl New_Viewer stuff end, listed for reference





%% WINGS_WM.erl Original Patch Section 2 Start, listed for Reference
%%+	#win{props=Props0} = Data = get_window_data(Win),
%%+	Props = gb_trees:enter(Name, Value, Props0),
%%+	put_window_data(Win, Data#win{props=Props}),
%%+
%+	%io:format("setting ~w on ~w to ~w\n", [Name, WindowName, Value]),
%%+	case Name of
%%+		stereo_partner ->
%%+			%io:format("stereo_partner, not double-setting\n");
%%+			ok;	
%%+		current_view ->
%%+			%io:format("current_view, not double-setting\n");
%%+			ok;
%%+		_Other ->
%%+
%%+			% link to other window 
%%+			case has_stereo_link(Win) of 
%%+				true ->
%%+					% get the other window
%%+					Other = get_stereo_link(Win),
%%+					WinOther = element(2,Other),
%%+					#win{props=Props0Other} = DataOther = get_window_data(WinOther),
%%+					PropsOther = gb_trees:enter(Name, Value, Props0Other),
%%+					put_window_data( WinOther, DataOther#win{props=PropsOther});
%%+				false ->
%%+					ok
%%+			end
%%+	end. 
%% WINGS_WM.erl Original Patch Section 2 End, listed for Reference.



%% WINGS_WM.erl Modified Patch Section 2 Start added by oort
%% This gets exported Third
set_prop(Win, Name, Value) ->
	#win{props=Props0} = Data = get_window_data(Win),
	Props = gb_trees:enter(Name, Value, Props0),
	put_window_data(Win, Data#win{props=Props}),

	io:format("Setting ~w for ~w\n", [Win, Name]),
	case Name of
		stereo_partner ->
			io:format("stereo_partner, not double-setting\n"),
			ok;	
		current_view ->
			io:format("current_view, not double-setting\n"),
			ok;
		_Other ->
			io:format("Other, not double-setting\n"),
			% link to other window 
			case has_stereo_link(Win) of
				true ->
					% get the other window
					Other = get_stereo_link(Win),
io:format("Other is ~w\n",[Other]),					
					WinOther = element(2,Other),
io:format("WinOther is ~w\n",[WinOther]),
					#win{props=Props0Other} = DataOther = get_window_data(WinOther),
					PropsOther = gb_trees:enter(Name, Value, Props0Other),
					put_window_data( WinOther, DataOther#win{props=PropsOther});
				false ->
					ok
				
			end

	end.     
%% WINGS_WM.erl Modified Patch Section 2 End added by oort

%% WINGS_WM.erl More stuff Added Start by oort
get_window_data(Name) ->
    gb_trees:get(Name, get(wm_windows)).

put_window_data(Name, Data) ->
    put(wm_windows, gb_trees:update(Name, Data, get(wm_windows))).
%% WINGS_WM.erl More stuff Added End by oort



%% WINGS_WM.erl Original Patch Section 3 start, listed for Reference.
%%+% link Win1 and Win2 as a stereo pair
%%+stereo_link(Win1, Win2) ->
%%+	set_prop(Win1,stereo_partner,{right,Win2}),
%%+	set_prop(Win2,stereo_partner,{left,Win1}).
%%+
%%+% is there a stereo link?
%%+has_stereo_link() ->
%%+	has_stereo_link(this()).
%%+has_stereo_link(Win) ->
%%+	Test = lookup_prop(Win, stereo_partner),
%%+	case Test of
%%+		none ->
%%+			false;
%%+		_Other ->
%%+			true
%%+	end.
%%+
%%+% get the stereo link
%%+get_stereo_link() ->
%%+	get_stereo_link(this()).
%%+get_stereo_link(Win) ->
%%+	get_prop(Win, stereo_partner).
%% WINGS_WM.erl Original Patch Section 3 end, listed for Reference.


%% WINGS_WM.erl Modified Patch Section 3 start (Stereo Link code)
%% This gets exported Second.
% link Win1 and Win2 as a stereo pair
stereo_link(Win1, Win2) ->
io:format("Win1 = ~w\,Win2 = ~w\n",[Win1,Win2]),
	set_prop(Win1,stereo_partner,{right,Win2}),
	set_prop(Win2,stereo_partner,{left,Win1}).
% is there a stereo link?
has_stereo_link() ->
	has_stereo_link(wings_wm:this()).
has_stereo_link(Win) ->
	Test = lookup_prop(Win, stereo_partner),
	case Test of
		none ->
			false;
		_Other ->
			true
	end.
% get the stereo link
get_stereo_link() ->
	get_stereo_link(wings_wm:this()).
get_stereo_link(Win) ->
	get_prop(Win, stereo_partner).
%% WINGS_WM.erl Modified Patch Section 3 End (Stereo Link code)



%% WINGS_VIEW.erl Original Patch Start, listed for Reference (Need to figure out how to add)
%%+	%io:format("set_current(View): setting first\n"),
%%     wings_wm:set_prop(current_view, View),
%%+	% look for a stereo partner
%%+	case wings_wm:has_stereo_link() of 
%%+		true ->
%%+			% has partner: which side?
%%+			{Side,Partner} = wings_wm:get_stereo_link(),
%%+			PanX = View#view.pan_x,
%%+			%io:format("this is  ~w~n",[View]),
%%+			case Side of 
%%+				left ->
%%+					OtherView = View#view{pan_x=PanX+0.07};
%%+					%io:format("other is ~w~n",[OtherView]);
%%+				right ->
%%+					OtherView = View#view{pan_x=PanX-0.07}
%%+					%io:format("other is ~w~n",[OtherView])
%%+			end,
%%+			%io:format("set_current(View): setting partner\n"),
%%+			wings_wm:set_prop(Partner,current_view,OtherView);
%%+		false ->
%%+			ok
%%+	end,
%%     View.
%% WINGS_VIEW.erl Original Patch End, listed for Reference. (Need to figure out how to add)
    

    

%% WINGS_CAMERA.erl Original Patch Section 1 & 2 Start, listed for Reference (Not needed)
%%-    stop_camera(Camera);
%%+	%io:format("camera released\n"),
%%+	%io:format("~w\n",[wings_view:current()]),
%%+	%io:format("windows: ~w\n",[wings_wm:windows()]),
%%+	%io:format("window properties: ~w\n",[wings_wm:get_props(wings_wm:this())]),
%%+	stop_camera(Camera);
%% nendo_event(#mousemotion{x=X,y=Y,state=Buttons}, Camera0, Redraw, true) ->
%%+	%io:format("camera moved ~B ~B\n",[X,Y]),

%% WINGS_CAMERA.erl Original Patch Section 1 & 2 End, listed for Reference (Not needed)






get_pref(Key, Def) ->
    wpa:pref_get(?MODULE, Key, Def).